@@ -496,7 +496,12 @@ def u_des_prep(B, C, u_order, u_lags, coig_data, T0_tot, M, constant, index,
 
 
 def e_des_prep(B, C, P, e_order, e_lags, res, sc_pred, Y_donors, out_feat, J, index,
-               index_w, coig_data, T0, T1, constant, e_design, outcome_var, P_diff_pre):
+               index_w, coig_data, T0, T1, constant, e_design, outcome_var, P_diff_pre,
+               effect, iota):
+
+    P, selp = trendRemove(P)
+    C, selc = trendRemove(C)
+    index = numpy.array(index)[numpy.array(selp)].tolist()
 
     Z = pandas.concat([B, C], axis=1)
 
@@ -505,7 +510,10 @@ def e_des_prep(B, C, P, e_order, e_lags, res, sc_pred, Y_donors, out_feat, J, in
 
         if coig_data is True:
             e_des_0 = Y_donors.loc[:, index_w] - Y_donors.loc[:, index_w].shift(1)
-            P_first = P.iloc[[0], :J] - Y_donors.iloc[[len(Y_donors) - 1], :].values
+            if effect == "time":
+                P_first = (P.iloc[[0], :J] * iota - Y_donors.iloc[[len(Y_donors) - 1], :].values) / iota
+            else:
+                P_first = P.iloc[[0], :J] - Y_donors.iloc[[len(Y_donors) - 1], :].values
             P_diff = P.iloc[:, :J].diff()
             P_diff.iloc[0, :] = P_first
             e_des_1 = P_diff.loc[:, index_w]
@@ -531,7 +539,10 @@ def e_des_prep(B, C, P, e_order, e_lags, res, sc_pred, Y_donors, out_feat, J, in
                 e_des_0 = pandas.concat([B_diff, C], axis=1).loc[:, index]
 
                 # Remove last observation of first feature from first period of P
-                P_first = P.iloc[[0], :J] - B.iloc[T0 - 1, :].values
+                if effect == "time":
+                    P_first = (P.iloc[[0], :J] * iota - B.iloc[T0 - 1, :].values) / iota
+                else:
+                    P_first = P.iloc[[0], :J] - B.iloc[T0 - 1, :].values
                 P_diff = P.iloc[:, :J].diff()
                 P_diff.iloc[0, :] = P_first
                 e_des_1 = pandas.concat([P_diff.loc[:, index_w],
@@ -998,7 +1009,7 @@ def prepareOptions(opt_dict, prompt):
     return opt_dict
 
 
-def scpi_out(y, x, preds, e_method, alpha, e_lb_est, e_ub_est):
+def scpi_out(y, x, preds, e_method, alpha, e_lb_est, e_ub_est, effect):
     e_1 = e_2 = None
     idx = preds.index
     y = deepcopy(numpy.array(y))[:, 0]
@@ -1011,13 +1022,30 @@ def scpi_out(y, x, preds, e_method, alpha, e_lb_est, e_ub_est):
             x_more = numpy.vstack((preds, x))
             fit = cond_pred(y=y, x=x, xpreds=x_more, method='lm')
             e_mean = fit[:len(preds)]
+            if effect == "time":
+                e_mean = pandas.DataFrame(data=e_mean, index=idx)
+                e_mean = e_mean.mean(level='__time').values[:, 0]
+
             y_fit = fit[len(preds):]
             y_var = numpy.log((y - y_fit)**2)
             var_pred = cond_pred(y=y_var, x=x, xpreds=x_more, method='lm')
-            e_sig2 = numpy.exp(var_pred[:len(preds)])
+            if effect == "time":
+                var_pred = pandas.DataFrame(data=var_pred[:len(preds)], index=idx)
+                var_pred = var_pred.mean(level='__time').values[:, 0]
+            else:
+                var_pred = var_pred[:len(preds)]
+            e_sig2 = numpy.exp(var_pred)
 
             q_pred = cond_pred(y=y - y_fit, x=x, xpreds=x_more, method='qreg', tau=[0.25, 0.75])
-            IQ_pred = q_pred[:len(preds), 1] - q_pred[:len(preds), 0]
+            if effect == "time":
+                q3_pred = pandas.DataFrame(data=q_pred[:len(preds), 1], index=idx)
+                q1_pred = pandas.DataFrame(data=q_pred[:len(preds), 0], index=idx)
+                q3_pred = q3_pred.mean(level='__time').values[:, 0]
+                q1_pred = q1_pred.mean(level='__time').values[:, 0]
+            else:
+                q3_pred = q_pred[:len(preds), 1]
+                q1_pred = q_pred[:len(preds), 0]
+            IQ_pred = q3_pred - q1_pred
             IQ_pred = numpy.absolute(IQ_pred)
             e_sig = numpy.c_[numpy.sqrt(e_sig2), IQ_pred / 1.34].min(axis=1)
 
@@ -1036,15 +1064,34 @@ def scpi_out(y, x, preds, e_method, alpha, e_lb_est, e_ub_est):
             x_more = numpy.vstack((preds, x))
             fit = cond_pred(y=y, x=x, xpreds=x_more, method='lm')
             e_mean = fit[:len(preds)]
+            if effect == "time":
+                e_mean = pandas.DataFrame(data=e_mean, index=idx)
+                e_mean = e_mean.mean(level='__time').values[:, 0]
+
             y_fit = fit[len(preds):]
             y_var = numpy.log((y - y_fit)**2)
             var_pred = cond_pred(y=y_var, x=x, xpreds=x_more, method='lm')
-            q_pred = cond_pred(y=y - y_fit, x=x, xpreds=preds, method='qreg', tau=[0.25, 0.75])
-            IQ_pred = q_pred[:len(preds), 1] - q_pred[:len(preds), 0]
+            res_var = var_pred[len(preds):]
+            if effect == "time":
+                var_pred = pandas.DataFrame(data=var_pred[:len(preds)], index=idx)
+                var_pred = var_pred.mean(level='__time').values[:, 0]
+            else:
+                var_pred = var_pred[:len(preds)]
+
+            q_pred = cond_pred(y=y - y_fit, x=x, xpreds=x_more, method='qreg', tau=[0.25, 0.75])
+            if effect == "time":
+                q3_pred = pandas.DataFrame(data=q_pred[:len(preds), 1], index=idx)
+                q1_pred = pandas.DataFrame(data=q_pred[:len(preds), 0], index=idx)
+                q3_pred = q3_pred.mean(level='__time').values[:, 0]
+                q1_pred = q1_pred.mean(level='__time').values[:, 0]
+            else:
+                q3_pred = q_pred[:len(preds), 1]
+                q1_pred = q_pred[:len(preds), 0]
+            IQ_pred = q3_pred - q1_pred
             IQ_pred = numpy.absolute(IQ_pred)
             e_sig = numpy.sqrt(numpy.exp(var_pred[:len(preds)]))
             e_sig = numpy.c_[e_sig, IQ_pred / 1.34].min(axis=1)
-            y_st = (y - y_fit) / numpy.sqrt(numpy.exp(var_pred[len(preds):]))
+            y_st = (y - y_fit) / numpy.sqrt(numpy.exp(res_var))
 
             lb = e_mean + e_sig * numpy.quantile(y_st, q=alpha)
             ub = e_mean + e_sig * numpy.quantile(y_st, q=(1 - alpha))
@@ -1058,8 +1105,20 @@ def scpi_out(y, x, preds, e_method, alpha, e_lb_est, e_ub_est):
         elif e_method == 'qreg':
             e_pred = cond_pred(y=y, x=x, xpreds=preds,
                                method='qreg', tau=[alpha, 1 - alpha])
-            lb = e_pred[:, [0]]
-            ub = e_pred[:, [1]]
+            if effect == "time":
+                e_pred_lb = pandas.DataFrame(data=e_pred[:, 0], index=idx)
+                e_pred_ub = pandas.DataFrame(data=e_pred[:, 1], index=idx)
+                e_pred_lb = e_pred_lb.mean(level='__time').values[:, 0]
+                e_pred_ub = e_pred_ub.mean(level='__time').values[:, 0]
+            else:
+                e_pred_lb = e_pred[:, [0]]
+                e_pred_ub = e_pred[:, [1]]
+
+            lb = e_pred_lb
+            ub = e_pred_ub
+
+        if effect == "time":
+            idx = idx.unique('__time')
 
         lb = pandas.DataFrame(lb, index=idx)
         ub = pandas.DataFrame(ub, index=idx)
@@ -1072,14 +1131,14 @@ def scpi_out(y, x, preds, e_method, alpha, e_lb_est, e_ub_est):
 
 
 def simultaneousPredGet(vsig, T1, T1_tot, iota, u_alpha, e_alpha, e_res_na, e_des_0_na,
-                        e_des_1, w_lb_est, w_ub_est, w_bounds, w_name):
+                        e_des_1, w_lb_est, w_ub_est, w_bounds, w_name, effect):
 
     vsigLB = vsig[:, :T1_tot]
     vsigUB = vsig[:, T1_tot:]
 
     e_lb, e_ub, e_1, e_2 = scpi_out(y=e_res_na, x=e_des_0_na, preds=e_des_1,
                                     e_method="gaussian", alpha=e_alpha / 2,
-                                    e_lb_est=True, e_ub_est=True)
+                                    e_lb_est=True, e_ub_est=True, effect=effect)
 
     jmin = 0
     w_lb_joint = []
@@ -1087,7 +1146,7 @@ def simultaneousPredGet(vsig, T1, T1_tot, iota, u_alpha, e_alpha, e_res_na, e_de
     for i in range(iota):
         jmax = T1[i] + jmin
 
-        if w_name in ['simplex', 'ridge', 'ols', 'L1-L2']:
+        if w_name in ['simplex', 'ridge', 'ols', 'L1-L2'] and effect != "time":
             lb_joint = numpy.nanquantile(numpy.nanquantile(vsigLB[:, jmin:jmax], q=0.05, axis=0),
                                          q=(u_alpha / 2), axis=0)
             ub_joint = numpy.nanquantile(numpy.nanquantile(vsigUB[:, jmin:jmax], q=0.95, axis=0),
@@ -1347,11 +1406,28 @@ def CIrename(ci, citype):
 
     return CI
 
-def detectConstant(x, tr):
-    n = len(x)
+def detectConstant(x, tr, scale_x=1):
+    x = x * scale_x
+    n = len(x.loc[complete_cases(x), ])
     col_keep = x.sum(axis=0) != n
     col_keep = numpy.logical_and(col_keep, (x.sum(axis=0) != 0))
     x = deepcopy(x.loc[:, col_keep])
     x.insert(0, tr + "_constant", 1, allow_duplicates=True)
-
+    x = x / scale_x
     return x
+
+def trendRemove(xxx):
+    x = deepcopy(xxx)
+    sel = []
+    for c in x.columns.tolist():
+        cp = c.split('_')
+        if len(cp) < 3:
+            sel.append(True)
+        else:
+            if cp[2] == "trend":
+                sel.append(False)
+            else:
+                sel.append(True)
+    xx = x.loc[:, numpy.array(sel)]
+
+    return xx, sel
