@@ -29,38 +29,39 @@ def scdata(df,
            cointegrated_data=False,
            anticipation=0,
            constant=False,
+           verbose=True,
            report_missing=False):
 
     """
     Parameters
     ----------
-     df : pandas.DataFrame
+    df : pandas.DataFrame
         a dataframe object containing the data to be processed
 
-     id_var : str
+    id_var : str
         a character with the name of the variable containing units' IDs
 
-     time_var : str
+    time_var : str
         a character with the name of the time variable. The time variable has to be numpy.int64, or one of
         pandas.Timestamp and numpy.datetime64. Input a numeric time variable is suggested when working with
         yearly data, whereas for all other frequencies either pandas.Timestamp or numpy.datetime64 types are preferred.
 
-     outcome_var : str
+    outcome_var : str
         a character with the name of the outcome variable. The outcome variable has to be numeric.
 
-     period_pre : array
+    period_pre : array
         a numeric vector that identifies the pre-treatment period in time_var.
 
-     period_post : array
+    period_post : array
          a numeric vector that identifies the post-treatment period in time_var.
 
-     unit_tr : int
+    unit_tr : int
          a scalar that identifies the treated unit in id_var.
 
-     unit_co : array
+    unit_co : array
          a numeric vector that identifies the donor pool in id_var.
 
-     features : list
+    features : list
          a character list containing the name of the feature variables used for estimation.
         If this option is not specified the default is features = outcome_var.
 
@@ -73,17 +74,20 @@ def scdata(df,
         feature, and so on. Finally, the user can specify 'constant' and 'trend' as covariates even if they are not
         present in the loaded dataframe.
 
-     constant : bool, default False
+    constant : bool, default False
          a logical which controls the inclusion of a constant term across features. The default value is False.
 
-     cointegrated_data : bool, default False
+    cointegrated_data : bool, default False
         a logical that indicates if there is a belief that the data is cointegrated or not. The default value is False.
         See the Details section for more.
 
-     anticipation : int
+    anticipation : int, default 0
         a scalar that indicates the number of periods of potential anticipation effects. Default is 0.
 
-     report_missing : bool, default False
+    verbose : bool, default True
+        a logical to print additional information in the console.
+
+    report_missing : bool, default False
         a logical which prints the location of missing values if present. The default value is False.
 
     Returns
@@ -135,6 +139,15 @@ def scdata(df,
     T1_outcome : int
         the number of post-treatment periods
 
+    features : list
+        a list with the name of the features
+
+    out_in_features : bool
+        for internal use only
+
+    effect : str
+        for internal use only
+
     glob_cons : bool
         for internal use only
 
@@ -154,7 +167,7 @@ def scdata(df,
 
     See Also
     --------
-    scest, scpi, scplot
+    scdataMulti, scest, scpi, scplot, scplotMulti
 
     """
     # Check main input is a dataframe
@@ -230,8 +243,10 @@ def scdata(df,
     else:
         data.rename(columns={time_var: '__time'}, inplace=True)
 
+    data['treated_unit'] = unit_tr
+
     # Create index
-    data.set_index(['__ID', '__time'], drop=False, inplace=True)
+    data.set_index(['treated_unit', '__ID', '__time'], drop=False, inplace=True)
 
     dd = data.iloc[0, data.columns.get_loc('__time')]
     if not isinstance(dd, (numpy.int64, numpy.int32, numpy.int16, pandas.Timestamp, numpy.datetime64)):
@@ -333,10 +348,10 @@ def scdata(df,
     data_bal['__ID'] = data_bal.index.get_level_values('__ID')
     data_bal['__time'] = data_bal.index.get_level_values('__time')
 
-    rows_tr_pre = data_bal.loc[(unit_tr_list, period_pre), ]
-    rows_tr_post = data_bal.loc[(unit_tr_list, period_post), ]
-    rows_co_pre = data_bal.loc[(unit_co, period_pre), ]
-    rows_co_post = data_bal.loc[(unit_co, period_post), ]
+    rows_tr_pre = data_bal.loc[(unit_tr, unit_tr_list, period_pre), ]
+    rows_tr_post = data_bal.loc[(unit_tr, unit_tr_list, period_post), ]
+    rows_co_pre = data_bal.loc[(unit_tr, unit_co, period_pre), ]
+    rows_co_post = data_bal.loc[(unit_tr, unit_co, period_post), ]
 
     ############################################################################
     # Estimation Data
@@ -352,7 +367,7 @@ def scdata(df,
 
     # Create A
     A_df = pandas.melt(rows_tr_pre,
-                       id_vars=['__ID', '__time'],
+                       id_vars=['treated_unit', '__ID', '__time'],
                        value_vars=features,
                        var_name='feature',
                        value_name=label)
@@ -363,22 +378,23 @@ def scdata(df,
 
     # Stack features one on top of the other
     B_df = pandas.melt(rows_co_pre,
-                       id_vars=['__ID', '__time'],
+                       id_vars=['treated_unit', '__ID', '__time'],
                        value_vars=features,
                        var_name='feature',
                        value_name='value')
 
     # make the df wide so that countries are one next to the other
-    B = B_df.pivot(index=['feature', '__time'],  # index to keep fixed
-                   columns='__ID',                # dimension to wide
-                   values='value')               # values to be widen
+    B = B_df.pivot(index=['treated_unit', 'feature', '__time'],   # index to keep fixed
+                   columns='__ID',                                # dimension to wide
+                   values='value')                                # values to be widen
 
     # Create list that dictates order of donors
     donor_order = B.columns.values.tolist()
 
     # Create matrix with outcome for the donors
-    sel = rows_co_pre[[outcome_var, '__time', '__ID']]
-    Y_donors = sel.pivot(index='__time', columns='__ID', values=outcome_var)
+    sel = rows_co_pre[[outcome_var, 'treated_unit', '__time', '__ID']]
+    Y_donors = sel.pivot(index=['treated_unit', '__time'],
+                         columns='__ID', values=outcome_var)
     Y_donors = Y_donors[donor_order]
 
     # Create C
@@ -401,7 +417,7 @@ def scdata(df,
                 C['trend'] = period_pre - period_pre[0] + 1
 
             # Add other covariates from dataframe
-            rows_C = data_bal.loc[(unit_co[0], period_pre), covs_adj].reset_index()
+            rows_C = data_bal.loc[(unit_tr, unit_co[0], period_pre), covs_adj].reset_index()
             C[covs_adj] = rows_C[covs_adj]
 
             for num in range(1, len(features) + 1):
@@ -428,7 +444,7 @@ def scdata(df,
                     C_m['trend'] = period_pre - period_pre[0] + 1
 
                 # Add other covariates from dataframe
-                rows_C = data_bal.loc[(unit_co[0], period_pre), covs_adj].reset_index()
+                rows_C = data_bal.loc[(unit_tr, unit_co[0], period_pre), covs_adj].reset_index()
                 C_m[covs_adj] = rows_C[covs_adj]
 
                 if m == 0:
@@ -475,9 +491,10 @@ def scdata(df,
     Y_post.columns = unit_tr_list
 
     # Prediction matrix
-    P = rows_co_post[[outcome_var, '__ID', '__time']].pivot(index='__time',
-                                                            columns='__ID',
-                                                            values=outcome_var)
+    P = rows_co_post[[outcome_var, 'treated_unit',
+                      '__ID', '__time']].pivot(index=['treated_unit', '__time'],
+                                               columns='__ID',
+                                               values=outcome_var)
 
     P = P[donor_order]
 
@@ -494,58 +511,55 @@ def scdata(df,
         # Add covariates used for adjustment in outcome variable equation (if present)
         if cov_adj is not None:
 
-            if not isinstance(cov_adj[0], list):
-                covs_adj = deepcopy(cov_adj)
-                if 'constant' in covs_adj:
-                    const = numpy.ones(len(rows_tr_post))
-                    const_n = str(1) + '_constant'
-                    P.insert(len(P.columns), const_n, const)
-                    covs_adj.remove('constant')
+            for m in range(1, len(features) + 1):
+                if not isinstance(cov_adj[0], list):
+                    covs_adj = deepcopy(cov_adj)
+                else:
+                    covs_adj = deepcopy(cov_adj[m - 1])
 
-                if 'trend' in covs_adj:
-                    trend = period_post - period_pre[0] + 1
-                    trend_n = str(1) + '_trend'
-                    P.insert(len(P.columns), trend_n, trend)
-                    covs_adj.remove('trend')
+                if m == 1:
 
-                # Add other covariates from dataframe
-                rows_P = data_bal.loc[(unit_co[0], period_post), covs_adj].reset_index()
-                rows_P.set_index(P.index, inplace=True)
-                coln = [str(1) + '_' + cov_n for cov_n in covs_adj]
-                P[coln] = rows_P[covs_adj]
+                    if 'constant' in covs_adj:
+                        const = numpy.ones(len(rows_tr_post))
+                        const_n = str(m) + '_constant'
+                        P.insert(len(P.columns), const_n, const)
+                        covs_adj.remove('constant')
 
-            else:
-                m = 1
-                for covs_adj in cov_adj:
-                    if m == 1:
-                        if 'constant' in covs_adj:
-                            const = numpy.ones(len(rows_tr_post))
-                            const_n = str(m) + '_constant'
-                            P.insert(len(P.columns), const_n, const)
-                            covs_adj.remove('constant')
+                    if 'trend' in covs_adj:
+                        trend = period_post - period_pre[0] + 1
+                        trend_n = str(m) + '_trend'
+                        P.insert(len(P.columns), trend_n, trend)
+                        covs_adj.remove('trend')
 
-                        if 'trend' in covs_adj:
-                            trend = period_post - period_pre[0] + 1
-                            trend_n = str(m) + '_trend'
-                            P.insert(len(P.columns), trend_n, trend)
-                            covs_adj.remove('trend')
+                    # Add other covariates from dataframe
+                    rows_P = data_bal.loc[(unit_tr, unit_co[0], period_post), covs_adj].reset_index()
+                    rows_P.set_index(P.index, inplace=True)
+                    coln = [str(m) + '_' + cov_n for cov_n in covs_adj]
+                    P[coln] = rows_P[covs_adj]
+                else:
+                    coln = [str(m) + '_' + cov_n for cov_n in covs_adj]
+                    zeros = numpy.zeros((len(P.index), len(covs_adj)))
+                    P[coln] = zeros
 
-                        # Add other covariates from dataframe
-                        rows_P = data_bal.loc[(unit_co[0], period_post), covs_adj].reset_index()
-                        rows_P.set_index(P.index, inplace=True)
-                        coln = [str(m) + '_' + cov_n for cov_n in covs_adj]
-                        P[coln] = rows_P[covs_adj]
-                    else:
-                        coln = [str(m) + '_' + cov_n for cov_n in covs_adj]
-                        zeros = numpy.zeros((len(P.index), len(covs_adj)))
-                        P[coln] = zeros
-
-                    m = m + 1
+                m = m + 1
 
     T1 = len(period_post)
 
     ############################################################################
     # Proceed cleaning missing data in the pre-treatment period
+    # Check if there are annoying donors with ALL missing values in the pre-treatment period
+    empty_cols = numpy.all(numpy.isnan(B), axis=0)
+
+    if sum(empty_cols) > 0:
+        names = B.columns[empty_cols].tolist()
+        names_print = ' '.join(str(name) for name in names)
+        if verbose is True:
+            warnings.warn("The following donors have no observations in the pre-treatment" +
+                          " period, hence they have been removed! " + names_print)
+        unit_co_eff = [co for co in unit_co if co not in names]
+        B = B[unit_co_eff]
+    else:
+        unit_co_eff = unit_co
 
     A.set_index(B.index, drop=True, inplace=True)
     if len(C.columns) > 0:
@@ -632,7 +646,7 @@ def scdata(df,
     # Store objects
 
     # Size of donor pool
-    J = len(unit_co)
+    J = len(unit_co_eff)
 
     # Total number of covariates used for adjustment
     if C is not None:
@@ -660,30 +674,45 @@ def scdata(df,
     K.columns = ['Feature', 'Num of Covariates']
     K = K.astype({'Feature': str, 'Num of Covariates': int})
 
-    return scdata_output(A=A_na, B=B_na, C=C_na, P=P, Y_pre=Y_pre,
+    A_na.columns = ['A']
+
+    B_na.columns = unit_tr + '_' + B_na.columns
+    if len(C_na.columns) > 0:
+        C_na.columns = unit_tr + '_' + C_na.columns
+
+    P.columns = unit_tr + '_' + P.columns
+    Y_donors.columns = unit_tr + '_' + Y_donors.columns
+
+    return scdata_output(A=A_na, B=B_na, C=C_na, P=P, P_diff=None, Y_pre=Y_pre,
                          Y_post=Y_post, Y_donors=Y_donors, J=J, K=K,
-                         KM=KM, M=M, cointegrated_data=cointegrated_data,
+                         KM=KM, KMI=KM, M=M, iota=1, cointegrated_data=cointegrated_data,
                          period_pre=period_pre, period_post=period_post,
                          T0_features=T0_features, T1_outcome=T1,
                          outcome_var=outcome_var, features=features,
-                         glob_cons=constant, out_in_features=out_in_features)
+                         glob_cons=constant, out_in_features=out_in_features,
+                         treated_units=[unit_tr], donors_units=unit_co_eff, units_est=[unit_tr],
+                         anticipation={unit_tr: anticipation}, effect="unit-time")
 
 class scdata_output:
-    def __init__(self, A, B, C, P, Y_pre, Y_post, Y_donors, J, K, KM, M,
+    def __init__(self, A, B, C, P, P_diff, Y_pre, Y_post, Y_donors, J, K, KM, KMI, M, iota,
                  cointegrated_data, period_pre, period_post, T0_features,
-                 T1_outcome, outcome_var, features, glob_cons, out_in_features):
+                 T1_outcome, outcome_var, features, glob_cons, out_in_features,
+                 treated_units, donors_units, units_est, anticipation, effect):
 
         self.A = A
         self.B = B
         self.C = C
         self.P = P
+        self.P_diff = P_diff
         self.Y_pre = Y_pre
         self.Y_post = Y_post
         self.Y_donors = Y_donors
         self.J = J
         self.K = K
         self.KM = KM
+        self.KMI = KMI
         self.M = M
+        self.iota = iota
         self.cointegrated_data = cointegrated_data
         self.period_pre = period_pre
         self.period_post = period_post
@@ -693,3 +722,8 @@ class scdata_output:
         self.features = features
         self.glob_cons = glob_cons
         self.out_in_features = out_in_features
+        self.treated_units = treated_units
+        self.donors_units = donors_units
+        self.units_est = units_est
+        self.anticipation = anticipation
+        self.effect = effect
