@@ -95,8 +95,9 @@ def scdataMulti(df,
 
     effect : str, default "unit-time"
         a string indicating the type of treatment effect to be estimated. Options are: 'unit-time', which estimates
-        treatment effects for each treated unit-time combination; 'unit', which estimates the treatment effect for
-        each unit by averaging post-treatment features over time.
+        treatment effects for each treated unit- post treatment period combination; 'unit', which estimates the
+        treatment effect for each unit by averaging post-treatment features over time; 'time', which estimates the
+        average treatment effect on the treated at various horizons.
 
     anticipation : int/dict, default 0
         a scalar that indicates the number of periods of potential anticipation effects. If the user wants to specify
@@ -215,8 +216,8 @@ def scdataMulti(df,
         if not isinstance(cov_adj, dict):
             raise Exception("The object 'cov_adj' should be a dictionary!")
 
-    if effect not in ["unit", "unit-time"]:
-        raise Exception("The object 'effect' should be either 'unit' or 'unit-time'!")
+    if effect not in ["unit", "unit-time", "time"]:
+        raise Exception("The object 'effect' should be either 'unit', 'time', or 'unit-time'!")
 
     # Check variables are in dataframe (id and time can be either variables or indices)
     if (id_var not in var_names) and (id_var not in indexes):
@@ -459,6 +460,13 @@ def scdataMulti(df,
         Y_donors_tr = scdata_out.Y_donors
         P_tr = scdata_out.P
 
+        if effect == "time":
+            time = P_tr.index.get_level_values('__time').tolist()
+            time = [t - min(time) + 1 for t in time]
+            P_tr = pandas.DataFrame(P_tr.values,
+                                    index=time,
+                                    columns=P_tr.columns)
+
         if not (effect == "unit" and cointegrated_data_tr is True):
             P_diff = None
 
@@ -519,7 +527,10 @@ def scdataMulti(df,
             A_stacked = pandas.concat([A_stacked, A_tr], axis=0)
             B_stacked = pandas.concat([B_stacked, B_tr], axis=0)
             C_stacked = pandas.concat([C_stacked, C_tr], axis=0)
-            P_stacked = pandas.concat([P_stacked, P_tr], axis=0)
+            if effect == "time":
+                P_stacked = pandas.concat([P_stacked, P_tr], axis=1, join='inner')  # stack horizontally
+            else:
+                P_stacked = pandas.concat([P_stacked, P_tr], axis=0)  # stack diagonally
             if Pd_stacked is not None:
                 Pd_stacked = pandas.concat([Pd_stacked, P_diff], axis=0)
             Y_donors_stacked = pandas.concat([Y_donors_stacked, Y_donors_tr], axis=0)
@@ -555,6 +566,9 @@ def scdataMulti(df,
     ccols = C_stacked.columns.tolist()
     P_stacked = P_stacked[bcols + ccols]
 
+    if effect == "time":
+        P_stacked = P_stacked / len(treated_units)
+
     # number of treated units and number of covariates used for adjustment
     iota = len(treated_units)
     KMI = len(C_stacked.columns)
