@@ -559,7 +559,15 @@ def scpi(data,
     else:
         C_dict = dict(zip(tr_units, [pandas.DataFrame(None)] * iota))
 
-    P_dict = mat2dict(P)
+    if sc_effect == "time":
+        P_dict = {}
+        for tr in tr_units:
+            csel = [c.split("_")[0] == tr for c in P.columns.tolist()]
+            X_rc = P.loc[:, numpy.array(csel)]
+            P_dict[tr] = X_rc
+    else:
+        P_dict = mat2dict(P)
+
     if sc_pred.P_diff is not None:
         Pd_dict = mat2dict(sc_pred.P_diff)
     else:
@@ -622,7 +630,8 @@ def scpi(data,
                                   e_lags, res_dict[tr], sc_pred, Yd_dict[tr],
                                   out_feat[tr], J[tr], index_i, iw_dict[tr],
                                   coig_data[tr], T0[tr][outcome_var], T1[tr],
-                                  constant[tr], e_design, outcome_var, Pd_dict[tr])
+                                  constant[tr], e_design, outcome_var, Pd_dict[tr],
+                                  sc_effect, iota)
 
         ###########################################################################
         # Remove NA - In sample uncertainty
@@ -660,16 +669,22 @@ def scpi(data,
         r_na.insert(0, "treated_unit", tr)
         er_na.insert(0, "treated_unit", tr)
         ed0_na.insert(0, "treated_unit", tr)
-        ed1.insert(0, "treated_unit", tr)
+        if sc_effect == "time":
+            idx = pandas.MultiIndex.from_product([[tr], ed1.index.get_level_values(0).tolist()],
+                                                 names=['treated_unit', '__time'])
+            ed1.set_index(idx, inplace=True, append=False)
+        else:
+            ed1.insert(0, "treated_unit", tr)
+            ed1.set_index('treated_unit', append=True, drop=True, inplace=True)
+            P_na.insert(0, "treated_unit", tr)
+            P_na.set_index('treated_unit', append=True, drop=True, inplace=True)
+
         Z_na.insert(0, "treated_unit", tr)
-        P_na.insert(0, "treated_unit", tr)
         ud0_na.set_index('treated_unit', append=True, drop=True, inplace=True)
         r_na.set_index('treated_unit', append=True, drop=True, inplace=True)
         er_na.set_index('treated_unit', append=True, drop=True, inplace=True)
         ed0_na.set_index('treated_unit', append=True, drop=True, inplace=True)
-        ed1.set_index('treated_unit', append=True, drop=True, inplace=True)
         Z_na.set_index('treated_unit', append=True, drop=True, inplace=True)
-        P_na.set_index('treated_unit', append=True, drop=True, inplace=True)
         u_des_0_na_dict[tr] = ud0_na
         res_na_dict[tr] = r_na
         e_res_na_dict[tr] = er_na
@@ -714,7 +729,8 @@ def scpi(data,
     e_res_na = e_res_na.reorder_levels(['treated_unit', '__time'])
     res_na = res_na.reorder_levels(['treated_unit', 'feature', '__time'])
     Z_na = Z_na.reorder_levels(['treated_unit', 'feature', '__time'])
-    P_na = P_na.reorder_levels(['treated_unit', '__time'])
+    if sc_effect != "time":
+        P_na = P_na.reorder_levels(['treated_unit', '__time'])
 
     Z_na = Z_na[col_order]
     P_na = P_na[col_order]
@@ -958,9 +974,14 @@ def scpi(data,
     e_des_0_na = pandas.DataFrame(None)
     e_des_1 = pandas.DataFrame(None)
 
+    if sc_effect == "time":
+        scale_x = sc_pred.iota
+    else:
+        scale_x = 1
+
     for tr in tr_units:
         ed0_dict[tr] = detectConstant(ed0_dict[tr], tr)
-        ed1_dict[tr] = detectConstant(ed1_dict[tr], tr)
+        ed1_dict[tr] = detectConstant(ed1_dict[tr], tr, scale_x)
         ed0_dict[tr].insert(0, "treated_unit", tr)
         ed0_dict[tr].set_index('treated_unit', append=False, drop=True, inplace=True)
         ed1_dict[tr].insert(0, "treated_unit", tr)
@@ -969,14 +990,21 @@ def scpi(data,
         e_des_1 = pandas.concat([e_des_1, ed1_dict[tr]], axis=0)
 
     e_des_0_na.set_index(e_res_na.index, append=False, inplace=True)
-    e_des_1.set_index(P_na.index, append=False, inplace=True)
+    if sc_effect == "time":
+        idx = pandas.MultiIndex.from_product([e_des_1.index.unique('treated_unit').tolist(),
+                                             [i for i in range(1, len(P_na) + 1)]],
+                                             names=['treated_unit', '__time'])
+        e_des_1.set_index(idx, append=False, inplace=True)
+    else:
+        e_des_1.set_index(P_na.index, append=False, inplace=True)
     e_des_0_na.fillna(0, inplace=True)
     e_des_1.fillna(0, inplace=True)
 
     if e_method == 'gaussian' or e_method == 'all':
         e_lb_gau, e_ub_gau, e_1, e_2 = scpi_out(y=e_res_na, x=e_des_0_na, preds=e_des_1,
                                                 e_method="gaussian", alpha=e_alpha / 2,
-                                                e_lb_est=e_lb_est, e_ub_est=e_ub_est)
+                                                e_lb_est=e_lb_est, e_ub_est=e_ub_est,
+                                                effect=sc_effect)
 
         # Overwrite with user's input
         if e_lb_est is False:
@@ -994,7 +1022,8 @@ def scpi(data,
     if e_method == 'ls' or e_method == 'all':
         e_lb_ls, e_ub_ls, e_1, e_2 = scpi_out(y=e_res_na, x=e_des_0_na, preds=e_des_1,
                                               e_method="ls", alpha=e_alpha / 2,
-                                              e_lb_est=e_lb_est, e_ub_est=e_ub_est)
+                                              e_lb_est=e_lb_est, e_ub_est=e_ub_est,
+                                              effect=sc_effect)
 
         # Overwrite with user's input
         if e_lb_est is False:
@@ -1013,7 +1042,8 @@ def scpi(data,
         else:
             e_lb_qreg, e_ub_qreg, e_1, e_2 = scpi_out(y=e_res_na, x=e_des_0_na, preds=e_des_1,
                                                       e_method="qreg", alpha=e_alpha / 2,
-                                                      e_lb_est=e_lb_est, e_ub_est=e_ub_est)
+                                                      e_lb_est=e_lb_est, e_ub_est=e_ub_est,
+                                                      effect=sc_effect)
         # Overwrite with user's input
         if e_lb_est is False:
             e_lb = e_bounds[0]
@@ -1032,20 +1062,32 @@ def scpi(data,
         ML, MU = simultaneousPredGet(vsig, T1val, len(P_na), iota, u_alpha,
                                      e_alpha, e_res_na, e_des_0_na, e_des_1,
                                      w_lb_est, w_ub_est, w_bounds,
-                                     w_constr_aux[tr_units[0]]['name'])
-    else:  # joint across units
+                                     w_constr_aux[tr_units[0]]['name'], sc_effect)
+
+    elif sc_effect == "unit":  # joint across units
         ML, MU = simultaneousPredGet(vsig, [len(P_na)], len(P_na), 1, u_alpha,
                                      e_alpha, e_res_na, e_des_0_na, e_des_1,
                                      w_lb_est, w_ub_est, w_bounds,
-                                     w_constr_aux[tr_units[0]]['name'])
+                                     w_constr_aux[tr_units[0]]['name'], sc_effect)
+
+    elif sc_effect == "time":  # joint within aggregate unit
+        ML, MU = simultaneousPredGet(vsig, [len(P_na)], len(P_na), 1, u_alpha,
+                                     e_alpha, e_res_na, e_des_0_na, e_des_1,
+                                     w_lb_est, w_ub_est, w_bounds,
+                                     w_constr_aux[tr_units[0]]['name'], sc_effect)
 
     ML.set_index(P_na.index, inplace=True, append=False)
     MU.set_index(P_na.index, inplace=True, append=False)
 
     ###############################################
     # Store all bounds
-    idx = w_lb.index
-    idx.rename(["ID", "Time"], inplace=True)
+    if sc_effect == "time":
+        idx = pandas.MultiIndex.from_product([["aggregate"], w_lb.index.get_level_values(0).tolist()],
+                                             names=['ID', 'Time'])
+    else:
+        idx = w_lb.index
+        idx.rename(["ID", "Time"], inplace=True)
+
     df_insample = pandas.DataFrame(numpy.c_[w_lb, w_ub], columns=["Lower", "Upper"], index=idx)
     df_insample = df_insample.apply(pandas.to_numeric, errors='coerce', axis=1)
 
@@ -1089,12 +1131,16 @@ def scpi(data,
 
     CI_0 = pandas.concat([sc_l_0, sc_r_0, len_0], axis=1)
     CI_0.columns = ['Lower', 'Upper', 'Length']
+    if sc_effect == "time":
+        CI_0.set_index(idx, inplace=True)
     CI_0.index.rename(['ID', 'Time'], inplace=True)
     CI_0 = CI_0.apply(pandas.to_numeric, errors='coerce', axis=1)
 
     if sc_l_1 is not None:
         CI_1 = pandas.concat([sc_l_1, sc_r_1, len_1], axis=1)
         CI_1.columns = ['Lower', 'Upper', 'Length']
+        if sc_effect == "time":
+            CI_1.set_index(idx, inplace=True)
         CI_1.index.rename(['ID', 'Time'], inplace=True)
         CI_1 = CI_1.apply(pandas.to_numeric, errors='coerce', axis=1)
     else:
@@ -1103,6 +1149,8 @@ def scpi(data,
     if sc_l_2 is not None:
         CI_2 = pandas.concat([sc_l_2, sc_r_2, len_2], axis=1)
         CI_2.columns = ['Lower', 'Upper', 'Length']
+        if sc_effect == "time":
+            CI_2.set_index(idx, inplace=True)
         CI_2.index.rename(['ID', 'Time'], inplace=True)
         CI_2 = CI_2.apply(pandas.to_numeric, errors='coerce', axis=1)
     else:
@@ -1111,6 +1159,8 @@ def scpi(data,
     if sc_l_3 is not None:
         CI_3 = pandas.concat([sc_l_3, sc_r_3, len_3], axis=1)
         CI_3.columns = ['Lower', 'Upper', 'Length']
+        if sc_effect == "time":
+            CI_3.set_index(idx, inplace=True)
         CI_3.index.rename(['ID', 'Time'], inplace=True)
         CI_3 = CI_3.apply(pandas.to_numeric, errors='coerce', axis=1)
     else:
