@@ -11,7 +11,7 @@ warnings.simplefilter(action='ignore', category=FutureWarning)
 import pandas
 import numpy
 from plotnine import ggplot, aes, geom_point, geom_errorbar, geom_vline, geom_line, theme, theme_bw
-from plotnine import element_blank, labs, guide_legend, scale_color_manual, ggtitle
+from plotnine import element_blank, labs, guide_legend, scale_color_manual, ggtitle, geom_ribbon
 
 
 def scplot(result,
@@ -22,6 +22,7 @@ def scplot(result,
            x_lab=None,
            y_lab=None,
            e_out=True,
+           joint=False,
            e_method=None,
            save_data=None):
 
@@ -53,6 +54,10 @@ def scplot(result,
     e_out : bool, default True
         a logical specifying whether out-of-sample uncertainty should be included in the plot(s).
 
+    joint : bool, default False
+        a logical specifying whether simultaneous prediction intervals should be included in the plot(s).
+        It requires e_out = True.
+
     e_method : str, default "gaussian"
         a string specifying the type of uncertainty estimation used for out-of-sample uncertainty quantification.
 
@@ -78,7 +83,7 @@ def scplot(result,
 
     See Also
     --------
-    scdata, scest, scpi
+    scdata, scdataMulti, scest, scpi, scplotMulti
 
     '''
     class_input = result.__class__.__name__
@@ -103,13 +108,13 @@ def scplot(result,
     T0 = period_pre[len(period_pre) - 1]
 
     if col_dots_t is None:
-        col_dots_t = 'gray'
+        col_dots_t = 'black'
     if col_line_t is None:
-        col_line_t = 'gray'
+        col_line_t = 'black'
     if col_dots_s is None:
-        col_dots_s = 'blue'
+        col_dots_s = 'mediumblue'
     if col_line_s is None:
-        col_line_s = 'blue'
+        col_line_s = 'mediumblue'
 
     # Store data for treated unit
     time = numpy.concatenate([period_pre, period_post])
@@ -126,7 +131,7 @@ def scplot(result,
         y_sc_na = pandas.DataFrame(numpy.array([numpy.nan] * len(time)))
 
         # Check if some periods have missing point estimate/missing CI
-        not_miss_plot = [t in y_sc_df.index.tolist() for t in time]
+        not_miss_plot = [t in y_sc_df.index.get_level_values(1).tolist() for t in time]
         y_sc_na.loc[not_miss_plot, ] = y_sc_df.iloc[:, [0]].to_numpy()
 
         data_points_act = pandas.DataFrame({'time': time,
@@ -150,14 +155,14 @@ def scplot(result,
                       )
 
         plot = (plot_struc +
-                geom_point(mapping=aes(x='time', y='y_act', color='tr'), shape='o', fill='white', na_rm=False)
-                + geom_point(mapping=aes(x='time', y='y_sc', color='sc'), shape='o', na_rm=False) +
+                geom_point(mapping=aes(x='time', y='y_act', color='tr'), shape='o', fill='white', na_rm=False) +
+                geom_point(mapping=aes(x='time', y='y_sc', color='sc'), shape='o', na_rm=False) +
                 geom_line(mapping=aes(x='time', y='y_act', color='tr'), na_rm=False) +
-                geom_line(mapping=aes(x='time', y='y_sc', color='sc'), linetype='dashed', na_rm=False) +
+                geom_line(mapping=aes(x='time', y='y_sc', color='sc'), na_rm=False) +
                 geom_vline(xintercept=T0, linetype='dotted') +
                 scale_color_manual(name="", values=[col_dots_t, col_dots_s],
                                    labels=["Treated", "Synthetic Control"],
-                                   guide=guide_legend(override_aes={'linetype': ['solid', 'dashed'],
+                                   guide=guide_legend(override_aes={'linetype': ['solid', 'solid'],
                                                                     'shape': ['o', 'o']})))
         return plot
 
@@ -192,16 +197,20 @@ def scplot(result,
         sc_r_0_na = pandas.DataFrame(numpy.array([numpy.nan] * len(time)))
         sc_l_1_na = pandas.DataFrame(numpy.array([numpy.nan] * len(time)))
         sc_r_1_na = pandas.DataFrame(numpy.array([numpy.nan] * len(time)))
+        sc_l_j_na = pandas.DataFrame(numpy.array([numpy.nan] * len(time)))
+        sc_r_j_na = pandas.DataFrame(numpy.array([numpy.nan] * len(time)))
 
         # Check if some periods have missing point estimate/missing CI
-        not_miss_plot = [t in y_sc_df.index.tolist() for t in time]
-        not_miss_ci = [t in result.CI_in_sample.index.tolist() for t in time]
+        not_miss_plot = [t in y_sc_df.index.get_level_values(1).tolist() for t in time]
+        not_miss_ci = [t in result.CI_in_sample.index.get_level_values(1).tolist() for t in time]
 
         y_sc_na.loc[not_miss_plot, ] = y_sc_df.iloc[:, [0]].to_numpy()
         sc_l_0_na.loc[not_miss_ci, ] = sc_l_0
         sc_r_0_na.loc[not_miss_ci, ] = sc_r_0
         sc_l_1_na.loc[not_miss_ci, ] = sc_l_1
         sc_r_1_na.loc[not_miss_ci, ] = sc_r_1
+        sc_l_j_na.loc[not_miss_ci, ] = sc_l_0 + result.bounds['joint'].iloc[:, [0]].to_numpy()
+        sc_r_j_na.loc[not_miss_ci, ] = sc_r_0 + result.bounds['joint'].iloc[:, [1]].to_numpy()
 
         data_points_act = pandas.DataFrame({'time': time,
                                             'y_act': y_act
@@ -209,8 +218,9 @@ def scplot(result,
 
         data_points = pandas.concat([data_points_act, y_sc_na,
                                     sc_l_0_na, sc_r_0_na,
-                                    sc_l_1_na, sc_r_1_na], axis=1)
-        data_points.columns = ['time', 'y_act', 'y_sc', 'lb0', 'ub0', 'lb1', 'ub1']
+                                    sc_l_1_na, sc_r_1_na,
+                                    sc_l_j_na, sc_r_j_na], axis=1)
+        data_points.columns = ['time', 'y_act', 'y_sc', 'lb0', 'ub0', 'lb1', 'ub1', 'lbj', 'ubj']
 
         data_points['tr'] = ['Treated'] * len(y_sc_na)
         data_points['sc'] = ['Synthetic Control'] * len(y_sc_na)
@@ -230,11 +240,11 @@ def scplot(result,
                       geom_point(mapping=aes(x='time', y='y_act', color='tr'), shape='o', fill='white', na_rm=False) +
                       geom_point(mapping=aes(x='time', y='y_sc', color='sc'), shape='o', na_rm=False) +
                       geom_line(mapping=aes(x='time', y='y_act', color='tr'), na_rm=False) +
-                      geom_line(mapping=aes(x='time', y='y_sc', color='sc'), linetype='dashed', na_rm=False) +
+                      geom_line(mapping=aes(x='time', y='y_sc', color='sc'), na_rm=False) +
                       geom_vline(xintercept=T0, linetype='dotted') +
                       scale_color_manual(name="", values=[col_dots_t, col_dots_s],
                                          labels=["Treated", "Synthetic Control"],
-                                         guide=guide_legend(override_aes={'linetype': ['solid', 'dashed'],
+                                         guide=guide_legend(override_aes={'linetype': ['solid', 'solid'],
                                                                           'shape': ['o', 'o']})))
 
         if e_out is False:
@@ -256,6 +266,9 @@ def scplot(result,
                 title_str = 'In and Out of Sample Uncertainty - Quantile Regression'
                 plot = plot_lines + geom_errorbar(mapping=aes(x='time', ymin='lb1', ymax='ub1', color='sc'),
                                                   size=0.5, linetype='solid') + ggtitle(title_str)
+            if joint is True:
+                plot = plot + geom_ribbon(mapping=aes(x='time', ymin='lbj', ymax='ubj'),
+                                          fill=col_dots_s, alpha=0.1)
 
         # Save data to reproduce plot
         if save_data is not None:
