@@ -111,6 +111,8 @@ def scplotMulti(result,
 
     plot_type = result.effect
     iota = result.iota
+    if plot_type == "time":
+        iota = 1
 
     if iota > 20 and (plot_type != "unit" or ptype != "treatment") and verbose:
         warnings.warn(str(iota) + " treated units detected, therefore some graphs might be too crowded!" +
@@ -121,8 +123,10 @@ def scplotMulti(result,
     Y_pre_fit = result.Y_pre_fit
     Y_post_fit = result.Y_post_fit
     synth_mat = pandas.concat([Y_pre_fit, Y_post_fit], axis=0)
-    synth_mat.index = synth_mat.index.rename(['ID', 'Time'])
-    synth_mat.columns = ['Synthetic']
+    if plot_type != "time":
+        synth_mat.index = synth_mat.index.rename(['ID', 'Time'])
+        synth_mat.columns = ['Synthetic']
+
     Y_df = result.Y_df
     Y_df.columns = ['ID', 'Time', 'Treatment', 'Actual']
 
@@ -155,9 +159,58 @@ def scplotMulti(result,
     sel_units = [i in result.units_est for i in treated_reception['ID']]
     treated_reception = treated_reception[sel_units]
 
-    toplot = pandas.concat([Y_actual, synth_mat], axis=1)
+    if plot_type == "time":
+        Y_actual_pre = res_df[res_df['Treatment'] == 0]
+        Y_actual_post = res_df[res_df['Treatment'] == 1]
+        tr_rec = deepcopy(treated_reception)
+        tr_rec['Tdate'] = tr_rec['Tdate'] + 1 / 2
+        Y_actual_pre = Y_actual_pre.merge(tr_rec[['ID', 'Tdate']], on="ID")
+        Y_actual_pre['tstd'] = Y_actual_pre['Time'] - Y_actual_pre['Tdate'] + 1
+        Y_actual_post = Y_actual_post.merge(tr_rec[['ID', 'Tdate']], on="ID")
+        Y_actual_post['tstd'] = Y_actual_post['Time'] - Y_actual_post['Tdate'] + 1
+
+        max_pre = max(Y_actual_pre.groupby(['ID'])['tstd'].min())
+        min_post = min(Y_actual_post.groupby(['ID'])['tstd'].max())
+
+        Y_actual_pre_agg = Y_actual_pre.groupby(['tstd'])['Actual'].mean()
+        Y_actual_post_agg = Y_actual_post.groupby(['tstd'])['Actual'].mean()
+
+        Y_actual_pre_agg = Y_actual_pre_agg.reset_index()
+        Y_actual_post_agg = Y_actual_post_agg.reset_index()
+
+        Y_actual_pre_agg = Y_actual_pre_agg[Y_actual_pre_agg['tstd'] >= max_pre]
+        Y_actual_post_agg = Y_actual_post_agg[Y_actual_post_agg['tstd'] <= min_post]
+
+        Y_actual_pre_agg['Treatment'] = 0
+        Y_actual_post_agg['Treatment'] = 1
+
+        Y_actual = pandas.concat([Y_actual_pre_agg, Y_actual_post_agg], axis=0)
+
+        synth_pre = deepcopy(Y_pre_fit.reset_index())
+        synth_post = deepcopy(Y_post_fit.reset_index())
+        synth_pre.rename(columns={"treated_unit": "ID", "__time": "Time", "A": "Synthetic"}, inplace=True)
+        synth_post.rename(columns={"index": "tstd", "A": "Synthetic"}, inplace=True)
+
+        synth_pre = synth_pre.merge(tr_rec[['ID', 'Tdate']], on="ID")
+        synth_pre['tstd'] = synth_pre['Time'] - synth_pre['Tdate'] + 1
+        synth_pre_agg = synth_pre.groupby(['tstd'])['Synthetic'].mean()
+        synth_pre_agg = synth_pre_agg.reset_index()
+        synth_mat = pandas.concat([synth_pre_agg, synth_post], axis=0)
+
+    if plot_type == "time":
+        toplot = Y_actual.merge(synth_mat)
+        toplot.rename(columns={"tstd": "Time", "Actual": "Actual",
+                               "Treatment": "Treatment", "Synthetic": "Synthetic"}, inplace=True)
+        toplot.insert(0, "ID", "aggregate")
+        treated_reception = pandas.DataFrame(data={'ID': 'aggregate', 'Tdate': 0}, index=[0])
+    else:
+        toplot = pandas.concat([Y_actual, synth_mat], axis=1)
+
     toplot['Effect'] = toplot['Actual'] - toplot['Synthetic']
-    toplot.reset_index(drop=False, inplace=True)
+    if plot_type != "time":
+        toplot.reset_index(drop=False, inplace=True)
+    else:
+        plot_type = "unit-time"
     toplot = toplot.merge(treated_reception[['ID', 'Tdate']], on='ID')
 
     if plot_type == 'unit-time' and ptype == "series":  # plot series
